@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005, 2006
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -44,26 +44,23 @@
 #include "global.h"
 #include "regex.h"
 #include "const.h"
+#include "filter.h"
 
 static void usage(void);
 static void help(void);
 static void setcom(int);
 int main(int, char **);
-void makefilter(STRBUF *);
-FILE *openfilter(void);
-void closefilter(FILE *);
+void printtag(const char *);
+void printtag_using(const char *, const char *, int, const char *);
 void completion(const char *, const char *, const char *);
 void idutils(const char *, const char *);
-void grep(const char *);
+void grep(const char *, const char *);
 void pathlist(const char *, const char *);
 void parsefile(int, char **, const char *, const char *, const char *, int);
-void printtag(FILE *, const char *);
-int search(const char *, const char *, const char *, int);
-int includepath(const char *, const char *);
-void ffformat(char *, int, const char *);
+int search(const char *, const char *, const char *, const char *, int);
+void tagsearch(const char *, const char *, const char *, const char *, int);
+void encode(char *, int, const char *);
 
-STRBUF *sortfilter;			/* sort filter		*/
-STRBUF *pathfilter;			/* path convert filter	*/
 const char *localprefix;		/* local prefix		*/
 int aflag;				/* [option]		*/
 int cflag;				/* command		*/
@@ -88,9 +85,13 @@ int xflag;				/* [option]		*/
 int show_version;
 int show_help;
 int show_filter;			/* undocumented command */
-int use_tagfiles;
+int nofilter;
+int nosource;				/* undocumented command */
 int debug;
-const char *extra_options;
+const char *gtags;
+int unique;
+int format;
+int passthru;
 
 static void
 usage(void)
@@ -107,17 +108,20 @@ help(void)
 	exit(0);
 }
 
+#define SHOW_FILTER	128
+#define RESULT		129
+
 static struct option const long_options[] = {
 	{"absolute", no_argument, NULL, 'a'},
 	{"completion", no_argument, NULL, 'c'},
 	{"regexp", required_argument, NULL, 'e'},
 	{"file", no_argument, NULL, 'f'},
 	{"local", no_argument, NULL, 'l'},
-	{"nofilter", no_argument, NULL, 'n'},
+	{"nofilter", optional_argument, NULL, 'n'},
 	{"grep", no_argument, NULL, 'g'},
 	{"basic-regexp", no_argument, NULL, 'G'},
 	{"ignore-case", no_argument, NULL, 'i'},
-	{"idutils", optional_argument, NULL, 'I'},
+	{"idutils", no_argument, NULL, 'I'},
 	{"other", no_argument, NULL, 'o'},
 	{"print-dbpath", no_argument, NULL, 'p'},
 	{"path", no_argument, NULL, 'P'},
@@ -135,14 +139,15 @@ static struct option const long_options[] = {
 	{"debug", no_argument, &debug, 1},
 	{"version", no_argument, &show_version, 1},
 	{"help", no_argument, &show_help, 1},
-	{"filter", no_argument, &show_filter, 1},
+	{"filter", optional_argument, NULL, SHOW_FILTER},
+	{"result", required_argument, NULL, RESULT},
+	{"nosource", no_argument, &nosource, 1},
 	{ 0 }
 };
 
 static int command;
 static void
-setcom(c)
-	int c;
+setcom(int c)
 {
 	if (command == 0)
 		command = c;
@@ -150,25 +155,19 @@ setcom(c)
 		usage();
 }
 int
-main(argc, argv)
-	int argc;
-	char *argv[];
+main(int argc, char **argv)
 {
 	const char *av = NULL;
-	int count;
 	int db;
 	int optchar;
 	int option_index = 0;
 	char cwd[MAXPATHLEN+1];			/* current directory	*/
 	char root[MAXPATHLEN+1];		/* root of source tree	*/
 	char dbpath[MAXPATHLEN+1];		/* dbpath directory	*/
-	const char *gtags;
 
 	while ((optchar = getopt_long(argc, argv, "ace:ifgGIlnopPqrstTuvx", long_options, &option_index)) != EOF) {
 		switch (optchar) {
 		case 0:
-			if (!strcmp("idutils", long_options[option_index].name))
-				extra_options = optarg;
 			break;
 		case 'a':
 			aflag++;
@@ -190,6 +189,14 @@ main(argc, argv)
 			break;
 		case 'n':
 			nflag++;
+			if (optarg) {
+				if (!strcmp(optarg, "sort"))
+					nofilter |= SORT_FILTER;
+				else if (!strcmp(optarg, "path"))
+					nofilter |= PATH_FILTER;
+			} else {
+				nofilter = BOTH_FILTER;
+			}
 			break;
 		case 'g':
 			gflag++;
@@ -242,6 +249,30 @@ main(argc, argv)
 		case 'x':
 			xflag++;
 			break;
+		case SHOW_FILTER:
+			if (optarg) {
+				if (!strcmp(optarg, "sort"))
+					show_filter = SORT_FILTER;
+				else if (!strcmp(optarg, "path"))
+					show_filter = PATH_FILTER;
+			} else {
+				show_filter = BOTH_FILTER;
+			}
+			break;
+		case RESULT:
+			if (!strcmp(optarg, "ctags-x"))
+				format = FORMAT_CTAGS_X;
+			else if (!strcmp(optarg, "ctags-xid"))
+				format = FORMAT_CTAGS_XID;
+			else if (!strcmp(optarg, "ctags"))
+				format = FORMAT_CTAGS;
+			else if (!strcmp(optarg, "path"))
+				format = FORMAT_PATH;
+			else if (!strcmp(optarg, "grep"))
+				format = FORMAT_GREP;
+			else
+				die_with_code(2, "unknown format type for the --result option.");
+			break;
 		default:
 			usage();
 			break;
@@ -269,7 +300,7 @@ main(argc, argv)
 	if (sflag && rflag)
 		die_with_code(2, "both of -s and -r are not allowed.");
 	/*
-	 * only -c, -i, -P and -p allows no argument.
+	 * only -c, -u, -P and -p allows no argument.
 	 */
 	if (!av && !show_filter) {
 		switch (command) {
@@ -299,6 +330,8 @@ main(argc, argv)
 		lflag = 0;
 	if (tflag)
 		xflag = 0;
+	if (nflag > 1)
+		nosource = 1;	/* to keep compatibility */
 	/*
 	 * remove leading blanks.
 	 */
@@ -326,7 +359,7 @@ main(argc, argv)
 		exit(0);
 	}
 	/*
-	 * incremental update of tag files.
+	 * get gtags's path.
 	 */
 	gtags = usable("gtags");
 	if (!gtags)
@@ -334,6 +367,9 @@ main(argc, argv)
 	gtags = strdup(gtags);
 	if (!gtags)
 		die("short of memory.");
+	/*
+	 * incremental update of tag files.
+	 */
 	if (uflag) {
 		STRBUF	*sb = strbuf_open(0);
 
@@ -350,7 +386,6 @@ main(argc, argv)
 		strbuf_close(sb);
 		exit(0);
 	}
-
 	/*
 	 * complete function name
 	 */
@@ -380,60 +415,57 @@ main(argc, argv)
 		localprefix = NULL;
 	}
 	/*
-	 * make sort filter.
+	 * Decide tag type.
 	 */
-	{
-		int unique = 0;
-		const char *sort;
-
-		/*
-		 * We cannot depend on the command PATH, because global(1)
-		 * might be called from WWW server. Since usable() looks for
-		 * the command in BINDIR directory first, gnusort need not
-		 * be in command PATH.
-		 */
-		sort = usable("gnusort");
-		if (!sort)
-			die("gnusort not found.");
-		sortfilter = strbuf_open(0);
-		strbuf_puts(sortfilter, sort);
-		if (sflag) {
-			strbuf_puts(sortfilter, " -u");
-			unique = 1;
-		}
-		if (tflag) 			/* ctags format */
-			strbuf_puts(sortfilter, " -k 1,1 -k 2,2 -k 3,3n");
-		else if (fflag)
-			strbuf_setlen(sortfilter, 0);
-		else if (xflag)			/* print details */
-			strbuf_puts(sortfilter, " -k 1,1 -k 3,3 -k 2,2n");
-		else if (!unique)		/* print just a file name */
-			strbuf_puts(sortfilter, " -u");
+	db = (rflag) ? GRTAGS : ((sflag) ? GSYMS : GTAGS);
+	unique = sflag ? 1 : 0;
+	/*
+	 * decide format.
+	 * The -t and -x option are given to priority more than
+	 * the --result option.
+	 */
+	if (fflag || gflag || Pflag || (nofilter & SORT_FILTER))
+		passthru = 1;
+	if (tflag) { 			/* ctags format */
+		format = FORMAT_CTAGS;
+	} else if (xflag) {		/* print details */
+		format = FORMAT_CTAGS_X;
+	} else if (format == 0) {	/* print just a file name */
+		format = FORMAT_PATH;
+		unique = 1;
 	}
 	/*
-	 * make path filter.
+	 * setup sort filter.
 	 */
-	pathfilter = strbuf_open(0);
-	strbuf_puts(pathfilter, gtags);
-	if (aflag)	/* absolute path name */
-		strbuf_puts(pathfilter, " --absolute");
-	else		/* relative path name */
-		strbuf_puts(pathfilter, " --relative");
-	if (xflag || tflag)
-		strbuf_puts(pathfilter, " --cxref");
-	strbuf_putc(pathfilter, ' ');
-	strbuf_puts(pathfilter, root);
-	strbuf_putc(pathfilter, ' ');
-	strbuf_puts(pathfilter, cwd);
+	setup_sortfilter(format, unique, passthru);
 	/*
-	 * print filter.
+	 * setup path filter.
+	 */
+	setup_pathfilter(format, aflag ? PATH_ABSOLUTE : PATH_RELATIVE, root, cwd, dbpath);
+	/*
+	 * print external filter.
 	 */
 	if (show_filter) {
-		STRBUF  *sb = strbuf_open(0);
+		STRBUF  *sb;
 
-		makefilter(sb);
-		fprintf(stdout, "%s\n", strbuf_value(sb));
-		strbuf_close(sb);
+		switch (show_filter) {
+		case SORT_FILTER:
+			if (!(nofilter & SORT_FILTER))
+				fprintf(stdout, "%s\n", getsortfilter());
+			break;
+		case PATH_FILTER:
+			if (!(nofilter & PATH_FILTER))
+				fprintf(stdout, "%s\n", getpathfilter());
+			break;
+		case BOTH_FILTER:
+			sb = strbuf_open(0);
+			makefilter(sb);
+			fprintf(stdout, "%s\n", strbuf_value(sb));
+			strbuf_close(sb);
+			break;
+		default:
+			die("internal error in show_filter.");
+		}
 		exit(0);
 	}
 	/*
@@ -442,138 +474,35 @@ main(argc, argv)
 	if (Iflag) {
 		chdir(root);
 		idutils(av, dbpath);
-		exit(0);
 	}
 	/*
-	 * grep the pattern in a source tree.
+	 * search pattern (regular expression).
 	 */
-	if (gflag) {
+	else if (gflag) {
 		chdir(root);
-		grep(av);
-		exit(0);
+		grep(av, dbpath);
 	}
 	/*
-	 * locate the path including the pattern in a source tree.
+	 * locate paths including the pattern.
 	 */
-	if (Pflag) {
+	else if (Pflag) {
 		chdir(root);
-		pathlist(dbpath, av);
-		exit(0);
+		pathlist(av, dbpath);
 	}
-	db = (rflag) ? GRTAGS : ((sflag) ? GSYMS : GTAGS);
 	/*
-	 * print function definitions.
+	 * parse source files.
 	 */
-	if (fflag) {
+	else if (fflag) {
 		parsefile(argc, argv, cwd, root, dbpath, db);
-		exit(0);
 	}
 	/*
-	 * search in current source tree.
+	 * tag search.
 	 */
-	count = search(av, root, dbpath, db);
-	/*
-	 * search in library path.
-	 */
-	if (getenv("GTAGSLIBPATH") && (count == 0 || Tflag) && !lflag && !rflag) {
-		STRBUF *sb = strbuf_open(0);
-		char libdbpath[MAXPATHLEN+1];
-		char *p, *lib;
-
-		strbuf_puts(sb, getenv("GTAGSLIBPATH"));
-		p = strbuf_value(sb);
-		while (p) {
-			lib = p;
-			if ((p = locatestring(p, PATHSEP, MATCH_FIRST)) != NULL)
-				*p++ = 0;
-			if (!gtagsexist(lib, libdbpath, sizeof(libdbpath), 0))
-				continue;
-			if (!strcmp(dbpath, libdbpath))
-				continue;
-			if (!test("f", makepath(libdbpath, dbname(db), NULL)))
-				continue;
-			strbuf_reset(pathfilter);
-			strbuf_puts(pathfilter, gtags);
-			if (aflag)	/* absolute path name */
-				strbuf_puts(pathfilter, " --absolute");
-			else		/* relative path name */
-				strbuf_puts(pathfilter, " --relative");
-			if (xflag || tflag)
-				strbuf_puts(pathfilter, " --cxref");
-			strbuf_putc(pathfilter, ' ');
-			strbuf_puts(pathfilter, lib);
-			strbuf_putc(pathfilter, ' ');
-			strbuf_puts(pathfilter, cwd);
-			count = search(av, lib, libdbpath, db);
-			if (count > 0 && !Tflag) {
-				strlimcpy(dbpath, libdbpath, sizeof(dbpath));
-				break;
-			}
-		}
-		strbuf_close(sb);
+	else {
+		tagsearch(av, cwd, root, dbpath, db);
 	}
-	if (vflag) {
-		if (count) {
-			if (count == 1)
-				fprintf(stderr, "%d object located", count);
-			if (count > 1)
-				fprintf(stderr, "%d objects located", count);
-		} else {
-			fprintf(stderr, "'%s' not found", av);
-		}
-		if (!Tflag)
-			fprintf(stderr, " (using '%s').\n", makepath(dbpath, dbname(db), NULL));
-	}
-	strbuf_close(sortfilter);
-	strbuf_close(pathfilter);
-
 	return 0;
 }
-/*
- * makefilter: make filter string.
- *
- *	o)	filter buffer
- */
-void
-makefilter(sb)
-	STRBUF *sb;
-{
-	if (!nflag) {
-		strbuf_puts(sb, strbuf_value(sortfilter));
-		if (strbuf_getlen(sortfilter) && strbuf_getlen(pathfilter))
-			strbuf_puts(sb, " | ");
-		strbuf_puts(sb, strbuf_value(pathfilter));
-	}
-}
-/*
- * openfilter: open output filter.
- *
- *	gi)	pathfilter
- *	gi)	sortfilter
- *	r)		file pointer for output filter
- */
-FILE *
-openfilter(void)
-{
-	FILE *op;
-	STRBUF *sb = strbuf_open(0);
-
-	makefilter(sb);
-	if (strbuf_getlen(sb) == 0)
-		op = stdout;
-	else
-		op = popen(strbuf_value(sb), "w");
-	strbuf_close(sb);
-	return op;
-}
-void
-closefilter(op)
-	FILE *op;
-{
-	if (op != stdout)
-		if (pclose(op) != 0)
-			die("terminated abnormally.");
-}
 /*
  * completion: print completion list of specified prefix
  *
@@ -582,10 +511,7 @@ closefilter(op)
  *	i)	prefix	prefix of primary key
  */
 void
-completion(dbpath, root, prefix)
-	const char *dbpath;
-	const char *root;
-	const char *prefix;
+completion(const char *dbpath, const char *root, const char *prefix)
 {
 	const char *p;
 	int flags = GTOP_KEY;
@@ -598,7 +524,7 @@ completion(dbpath, root, prefix)
 	if (prefix)
 		flags |= GTOP_PREFIX;
 	db = (sflag) ? GSYMS : GTAGS;
-	gtop = gtags_open(dbpath, root, db, GTAGS_READ, 0);
+	gtop = gtags_open(dbpath, root, db, GTAGS_READ);
 	for (p = gtags_first(gtop, prefix, flags); p; p = gtags_next(gtop)) {
 		fputs(p, stdout);
 		fputc('\n', stdout);
@@ -608,37 +534,67 @@ completion(dbpath, root, prefix)
 /*
  * printtag: print a tag's line
  *
- *	i)	op	output stream
- *	i)	line	tag's line
+ *	i)	ctags_x	ctags -x format record
  */
 void
-printtag(op, line)
-	FILE *op;
-	const char *line;		/* virtually const */
+printtag(const char *ctags_x)		/* virtually const */
 {
-	if (xflag) {
-		fputs(line, op);
-	} else {
-		SPLIT ptable;
-		int n;
+	SPLIT ptable;
 
-		/*
-		 * Split tag line.
-		 */
-		n = split((char *)line, 4, &ptable);
-
-		if (tflag) {
-			fputs(ptable.part[0].start, op);	/* tag */
-			(void)putc('\t', op);
-			fputs(ptable.part[2].start, op);	/* path */
-			(void)putc('\t', op);
-			fputs(ptable.part[1].start, op);	/* line number */
-		} else {
-			fputs(ptable.part[2].start, op);	/* path */
-		}
+	switch (format) {
+	case FORMAT_PATH:
+		split((char *)ctags_x, 4, &ptable);
+		printtag_using(
+			NULL,					/* tag */
+			ptable.part[PART_PATH].start,		/* path */
+			0,					/* line no */
+			NULL);
 		recover(&ptable);
+		break;
+	default:
+		filter_put(ctags_x);
+		break;
 	}
-	fputc('\n', op);
+}
+/*
+ * printtag_using: print a tag's line with arguments
+ *
+ *	i)	tag	tag name
+ *	i)	path	path name
+ *	i)	lineno	line number
+ *	i)	line	line image
+ */
+void
+printtag_using(const char *tag, const char *path, int lineno, const char *line)
+{
+	STATIC_STRBUF(sb);
+	char edit[MAXPATHLEN];
+
+	strbuf_clear(sb);
+	/*
+	 * normalize path name.
+	 */
+	if (*path != '.') {
+		int i = 0;
+		edit[i++] = '.';
+		edit[i++] = '/';
+		while ((edit[i++] = *path++) != '\0')
+			;
+		path = edit;
+	}
+	/*
+	 * The other format than FORMAT_PATH is always written as FORMAT_CTAGS_X.
+	 * Format conversion will be done in the following process.
+	 */
+	switch (format) {
+	case FORMAT_PATH:
+		strbuf_puts(sb, path);
+		break;
+	default:
+		strbuf_sprintf(sb, "%-16s %4d %-16s %s", tag, lineno, path, line);
+		break;
+	}
+	filter_put(strbuf_value(sb));
 }
 /*
  * idutils:  lid(id-utils) pattern
@@ -647,16 +603,14 @@ printtag(op, line)
  *	i)	dbpath	GTAGS directory
  */
 void
-idutils(pattern, dbpath)
-	const char *pattern;
-	const char *dbpath;
+idutils(const char *pattern, const char *dbpath)
 {
-	FILE *ip, *op;
+	FILE *ip;
 	STRBUF *ib = strbuf_open(0);
 	char edit[IDENTLEN+1];
 	const char *path, *lno, *lid;
 	int linenum, count;
-	char *p, *line;
+	char *p, *grep;
 
 	lid = usable("lid");
 	if (!lid)
@@ -664,9 +618,10 @@ idutils(pattern, dbpath)
 	/*
 	 * convert spaces into %FF format.
 	 */
-	ffformat(edit, sizeof(edit), pattern);
+	encode(edit, sizeof(edit), pattern);
 	/*
 	 * make lid command line.
+	 * Invoke lid with the --result=grep option to generate grep format.
 	 */
 	strbuf_puts(ib, lid);
 	strbuf_puts(ib, " --separator=newline");
@@ -676,73 +631,68 @@ idutils(pattern, dbpath)
 		strbuf_puts(ib, " --result=grep");
 	if (iflag)
 		strbuf_puts(ib, " --ignore-case");
-	if (extra_options) {
-		strbuf_putc(ib, ' ');
-		strbuf_puts(ib, extra_options);
-	}
 	strbuf_putc(ib, ' ');
 	strbuf_puts(ib, quote_string(pattern));
 	if (debug)
 		fprintf(stderr, "id-utils: %s\n", strbuf_value(ib));
 	if (!(ip = popen(strbuf_value(ib), "r")))
 		die("cannot execute '%s'.", strbuf_value(ib));
-	if (!(op = openfilter()))
-		die("cannot open output filter.");
+	filter_open();
 	count = 0;
-	while ((line = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL) {
-		p = line;
+	while ((grep = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL) {
+		p = grep;
 		/* extract filename */
 		path = p;
 		while (*p && *p != ':')
 			p++;
 		if ((xflag || tflag) && !*p)
-			die("invalid lid(id-utils) output format. '%s'", line);
+			die("invalid lid(id-utils) output format. '%s'", grep);
 		*p++ = 0;
 		if (lflag) {
 			if (!locatestring(path, localprefix + 2, MATCH_AT_FIRST))
 				continue;
 		}
 		count++;
-		if (!xflag && !tflag) {
-			fprintf(op, "./%s\n", path);
-			continue;
-		}
-		/* extract line number */
-		while (*p && isspace(*p))
-			p++;
-		lno = p;
-		while (*p && isdigit(*p))
-			p++;
-		if (*p != ':')
-			die("invalid lid(id-utils) output format. '%s'", line);
-		*p++ = 0;
-		linenum = atoi(lno);
-		if (linenum <= 0)
-			die("invalid lid(id-utils) output format. '%s'", line);
-		/*
-		 * print out.
-		 */
-		if (tflag)
-			fprintf(op, "%s\t./%s\t%d\n", edit, path, linenum);
-		else {
-			char	buf[MAXPATHLEN+1];
-
-			snprintf(buf, sizeof(buf), "./%s", path);
-			fprintf(op, "%-16s %4d %-16s %s\n",
-					edit, linenum, buf, p);
+		switch (format) {
+		case FORMAT_PATH:
+			printtag_using(NULL, path, 0, NULL);
+			break;
+		default:
+			/* extract line number */
+			while (*p && isspace(*p))
+				p++;
+			lno = p;
+			while (*p && isdigit(*p))
+				p++;
+			if (*p != ':')
+				die("invalid lid(id-utils) output format. '%s'", grep);
+			*p++ = 0;
+			linenum = atoi(lno);
+			if (linenum <= 0)
+				die("invalid lid(id-utils) output format. '%s'", grep);
+			/*
+			 * print out.
+			 */
+			printtag_using(edit, path, linenum, p);
+			break;
 		}
 	}
 	if (pclose(ip) < 0)
 		die("terminated abnormally.");
-	closefilter(op);
+	filter_close();
 	strbuf_close(ib);
 	if (vflag) {
-		if (count == 0)
+		switch (count) {
+		case 0:
 			fprintf(stderr, "object not found");
-		if (count == 1)
+			break;
+		case 1:
 			fprintf(stderr, "%d object located", count);
-		if (count > 1)
+			break;
+		default:
 			fprintf(stderr, "%d objects located", count);
+			break;
+		}
 		fprintf(stderr, " (using id-utils index in '%s').\n", dbpath);
 	}
 }
@@ -752,12 +702,12 @@ idutils(pattern, dbpath)
  *	i)	pattern	POSIX regular expression
  */
 void
-grep(pattern)
-	const char *pattern;
+grep(const char *pattern, const char *dbpath)
 {
-	FILE *op, *fp;
+	FILE *fp;
+	GFIND *gp;
 	STRBUF *ib = strbuf_open(MAXBUFLEN);
-	const char *path, *p;
+	const char *path;
 	char edit[IDENTLEN+1];
 	const char *buffer;
 	int linenum, count;
@@ -767,7 +717,7 @@ grep(pattern)
 	/*
 	 * convert spaces into %FF format.
 	 */
-	ffformat(edit, sizeof(edit), pattern);
+	encode(edit, sizeof(edit), pattern);
 
 	if (!Gflag)
 		flags |= REG_EXTENDED;
@@ -775,44 +725,41 @@ grep(pattern)
 		flags |= REG_ICASE;
 	if (regcomp(&preg, pattern, flags) != 0)
 		die("invalid regular expression.");
-	if (!(op = openfilter()))
-		die("cannot open output filter.");
+	filter_open();
 	count = 0;
-	for (vfind_open(localprefix, oflag); (p = vfind_read()) != NULL; ) {
-		path = (*p == ' ') ? ++p : p;
+
+	gp = gfind_open(dbpath, localprefix, oflag);
+	while ((path = gfind_read(gp)) != NULL) {
 		if (!(fp = fopen(path, "r")))
-			die("cannot open file '%s'.", path);
+			die("'%s' not found. Please remake tag files by invoking gtags(1).", path);
 		linenum = 0;
 		while ((buffer = strbuf_fgets(ib, fp, STRBUF_NOCRLF)) != NULL) {
 			linenum++;
 			if (regexec(&preg, buffer, 0, 0, 0) == 0) {
 				count++;
-				if (tflag)
-					fprintf(op, "%s\t%s\t%d\n",
-						edit, path, linenum);
-				else if (!xflag) {
-					fputs(path, op);
-					fputc('\n', op);
+				printtag_using(edit, path, linenum, buffer);
+				if (format == FORMAT_PATH)
 					break;
-				} else {
-					fprintf(op, "%-16s %4d %-16s %s\n",
-						edit, linenum, path, buffer);
-				}
 			}
 		}
 		fclose(fp);
 	}
-	vfind_close();
-	closefilter(op);
+	gfind_close(gp);
+	filter_close();
 	strbuf_close(ib);
 	regfree(&preg);
 	if (vflag) {
-		if (count == 0)
+		switch (count) {
+		case 0:
 			fprintf(stderr, "object not found");
-		if (count == 1)
+			break;
+		case 1:
 			fprintf(stderr, "%d object located", count);
-		if (count > 1)
+			break;
+		default:
 			fprintf(stderr, "%d objects located", count);
+			break;
+		}
 		fprintf(stderr, " (no index used).\n");
 	}
 }
@@ -822,17 +769,16 @@ grep(pattern)
  *	i)	dbpath
  */
 void
-pathlist(dbpath, av)
-	const char *dbpath;
-	const char *av;
+pathlist(const char *pattern, const char *dbpath)
 {
-	FILE *op;
+	GFIND *gp;
 	const char *path, *p;
 	regex_t preg;
 	int count;
 
-	if (av) {
+	if (pattern) {
 		int flags = 0;
+		char edit[IDENTLEN+1];
 
 		if (!Gflag)
 			flags |= REG_EXTENDED;
@@ -841,43 +787,48 @@ pathlist(dbpath, av)
 #ifdef _WIN32
 		flags |= REG_ICASE;
 #endif /* _WIN32 */
-		if (regcomp(&preg, av, flags) != 0)
+		/*
+		 * We assume '^aaa' as '^/aaa'.
+		 */
+		if (*pattern == '^' && *(pattern + 1) != '/') {
+			snprintf(edit, sizeof(edit), "^/%s", pattern + 1);
+			pattern = edit;
+		}
+		if (regcomp(&preg, pattern, flags) != 0)
 			die("invalid regular expression.");
 	}
 	if (!localprefix)
-		localprefix = ".";
-	if (!(op = openfilter()))
-		die("cannot open output filter.");
+		localprefix = "./";
+	filter_open();
 	count = 0;
-	for (vfind_open(localprefix, oflag); (p = vfind_read()) != NULL; ) {
-		path = (*p == ' ') ? ++p : p;
+
+	gp = gfind_open(dbpath, localprefix, oflag);
+	while ((path = gfind_read(gp)) != NULL) {
 		/*
 		 * skip localprefix because end-user doesn't see it.
 		 */
 		p = path + strlen(localprefix) - 1;
-		if (av && regexec(&preg, p, 0, 0, 0) != 0)
+		if (pattern && regexec(&preg, p, 0, 0, 0) != 0)
 			continue;
-		if (xflag)
-			fprintf(op, "path\t1 %s \n", path);
-		else if (tflag)
-			fprintf(op, "path\t%s\t1\n", path);
-		else {
-			fputs(path, op);
-			fputc('\n', op);
-		}
+		printtag_using("path", path, 1, " ");
 		count++;
 	}
-	vfind_close();
-	closefilter(op);
-	if (av)
+	gfind_close(gp);
+	filter_close();
+	if (pattern)
 		regfree(&preg);
 	if (vflag) {
-		if (count == 0)
+		switch (count) {
+		case 0:
 			fprintf(stderr, "path not found");
-		if (count == 1)
+			break;
+		case 1:
 			fprintf(stderr, "%d path located", count);
-		if (count > 1)
+			break;
+		default:
 			fprintf(stderr, "%d paths located", count);
+			break;
+		}
 		fprintf(stderr, " (using '%s').\n", makepath(dbpath, dbname(GPATH), NULL));
 	}
 }
@@ -892,42 +843,45 @@ pathlist(dbpath, av)
  *	i)	db	type of parse
  */
 void
-parsefile(argc, argv, cwd, root, dbpath, db)
-	int argc;
-	char **argv;
-	const char *cwd;
-	const char *root;
-	const char *dbpath;
-	int db;
+parsefile(int argc, char **argv, const char *cwd, const char *root, const char *dbpath, int db)
 {
+	char rootdir[MAXPATHLEN+1];
 	char buf[MAXPATHLEN+1], *path;
-	const char *p;
-	FILE *ip, *op;
-	const char *parser, *av;
-	int count;
-	STRBUF *sb = strbuf_open(0);
-	STRBUF *com = strbuf_open(0);
-	STRBUF *ib = strbuf_open(MAXBUFLEN);
+	int count = 0;
+	STRBUF *comline = strbuf_open(0);
+	STRBUF *path_list = strbuf_open(MAXPATHLEN);
+	XARGS *xp;
+	char *ctags_x;
 
+	snprintf(rootdir, sizeof(rootdir), "%s/", root);
 	/*
 	 * teach parser where is dbpath.
 	 */
 	set_env("GTAGSDBPATH", dbpath);
+	/*
+	 * teach parser language mapping.
+	 */
+	{
+		STRBUF *sb = strbuf_open(0);
 
+		if (getconfs("langmap", sb))
+			set_env("GTAGSLANGMAP", strbuf_value(sb));
+		strbuf_close(sb);
+	}
 	/*
 	 * get parser.
 	 */
-	if (!getconfs(dbname(db), sb))
+	if (!getconfs(dbname(db), comline))
 		die("cannot get parser for %s.", dbname(db));
-	parser = strbuf_value(sb);
-
-	if (!(op = openfilter()))
-		die("cannot open output filter.");
-	if (gpath_open(dbpath, 0, 0) < 0)
+	filter_open();
+	if (gpath_open(dbpath, 0) < 0)
 		die("GPATH not found.");
-	count = 0;
+	/*
+	 * Make a path list while checking the validity of path name.
+	 */
 	for (; argc > 0; argv++, argc--) {
-		av = argv[0];
+		const char *av = argv[0];
+
 		if (!test("f", av)) {
 			if (test("d", av)) {
 				if (!qflag)
@@ -946,50 +900,61 @@ parsefile(argc, argv, cwd, root, dbpath, db)
 			die("realpath(%s, buf) failed. (errno=%d).", av, errno);
 		if (!isabspath(path))
 			die("realpath(3) is not compatible with BSD version.");
-		if (strncmp(path, root, strlen(root))) {
+		/*
+		 * Remove the root part of path and insert './'.
+		 *      rootdir  /a/b/
+		 *      path     /a/b/c/d.c -> c/d.c -> ./c/d.c
+		 */
+		path = locatestring(path, rootdir, MATCH_AT_FIRST);
+		if (path == NULL) {
 			if (!qflag)
-				fprintf(stderr, "'%s' is out of source tree.\n", path);
+				fprintf(stderr, "'%s' is out of source tree.\n", buf);
 			continue;
 		}
-		path += strlen(root) - 1;
+		path -= 2;
 		*path = '.';
-		if (!gpath_path2fid(path)) {
+		if (!gpath_path2fid(path, NULL)) {
 			if (!qflag)
 				fprintf(stderr, "'%s' not found in GPATH.\n", path);
 			continue;
 		}
-		if (chdir(root) < 0)
-			die("cannot move to '%s' directory.", root);
 		/*
-		 * make command line.
+		 * Add a path to the path list.
 		 */
-		strbuf_reset(com);
-		makecommand(parser, path, com);
-		if (debug)
-			fprintf(stderr, "executing %s\n", strbuf_value(com));
-		if (!(ip = popen(strbuf_value(com), "r")))
-			die("cannot execute '%s'.", strbuf_value(com));
-		while ((p = strbuf_fgets(ib, ip, STRBUF_NOCRLF)) != NULL) {
-			count++;
-			printtag(op, p);
-		}
-		if (pclose(ip) < 0)
-			die("terminated abnormally.");
-		if (chdir(cwd) < 0)
-			die("cannot move to '%s' directory.", cwd);
+		strbuf_puts0(path_list, path);
 	}
+	/*
+	 * Execute parser in the root directory of source tree.
+	 */
+	if (chdir(root) < 0)
+		die("cannot move to '%s' directory.", root);
+	xp = xargs_open_with_strbuf(strbuf_value(comline), 0, path_list);
+	while ((ctags_x = xargs_read(xp)) != NULL) {
+		printtag(ctags_x);
+		count++;
+	}
+	xargs_close(xp);
+	if (chdir(cwd) < 0)
+		die("cannot move to '%s' directory.", cwd);
+	/*
+	 * Settlement
+	 */
 	gpath_close();
-	closefilter(op);
-	strbuf_close(sb);
-	strbuf_close(com);
-	strbuf_close(ib);
+	filter_close();
+	strbuf_close(comline);
+	strbuf_close(path_list);
 	if (vflag) {
-		if (count == 0)
+		switch (count) {
+		case 0:
 			fprintf(stderr, "object not found");
-		if (count == 1)
+			break;
+		case 1:
 			fprintf(stderr, "%d object located", count);
-		if (count > 1)
+			break;
+		default:
 			fprintf(stderr, "%d objects located", count);
+			break;
+		}
 		fprintf(stderr, " (no index used).\n");
 	}
 }
@@ -998,20 +963,16 @@ parsefile(argc, argv, cwd, root, dbpath, db)
  *
  *	i)	pattern		search pattern
  *	i)	root		root of source tree
+ *	i)	cwd		current directory
  *	i)	dbpath		database directory
  *	i)	db		GTAGS,GRTAGS,GSYMS
  *	r)			count of output lines
  */
 int
-search(pattern, root, dbpath, db)
-	const char *pattern;
-	const char *root;
-	const char *dbpath;
-	int db;
+search(const char *pattern, const char *root, const char *cwd, const char *dbpath, int db)
 {
-	const char *p;
+	const char *ctags_x;
 	int count = 0;
-	FILE *op;
 	GTOP *gtop;
 	int flags = 0;
 	STRBUF *sb = NULL;
@@ -1019,13 +980,12 @@ search(pattern, root, dbpath, db)
 	/*
 	 * open tag file.
 	 */
-	gtop = gtags_open(dbpath, root, db, GTAGS_READ, 0);
-	if (!(op = openfilter()))
-		die("cannot open output filter.");
+	gtop = gtags_open(dbpath, root, db, GTAGS_READ);
+	filter_open();
 	/*
 	 * search through tag file.
 	 */
-	if (nflag > 1)
+	if (nosource)
 		flags |= GTOP_NOSOURCE;
 	if (iflag) {
 		if (!isregex(pattern)) {
@@ -1039,60 +999,107 @@ search(pattern, root, dbpath, db)
 	}
 	if (Gflag)
 		flags |= GTOP_BASICREGEX;
-	for (p = gtags_first(gtop, pattern, flags); p; p = gtags_next(gtop)) {
+	for (ctags_x = gtags_first(gtop, pattern, flags); ctags_x; ctags_x = gtags_next(gtop)) {
 		if (lflag) {
 			const char *q;
 			/* locate start point of a path */
-			q = locatestring(p, "./", MATCH_FIRST);
+			q = locatestring(ctags_x, "./", MATCH_FIRST);
 			if (!locatestring(q, localprefix, MATCH_AT_FIRST))
 				continue;
 		}
-		printtag(op, p);
+		printtag(ctags_x);
 		count++;
 	}
-	closefilter(op);
+	filter_close();
 	if (sb)
 		strbuf_close(sb);
 	gtags_close(gtop);
 	return count;
 }
 /*
- * includepath: check if the path included in tag line or not.
+ * tagsearch: execute tag search
  *
- *	i)	line	tag line
- *	i)	path	path
- *	r)		0: doesn't included, 1: included
+ *	i)	pattern		search pattern
+ *	i)	cwd		current directory
+ *	i)	root		root of source tree
+ *	i)	dbpath		database directory
+ *	i)	db		GTAGS,GRTAGS,GSYMS
  */
-int
-includepath(line, path)
-	const char *line;
-	const char *path;
+void
+tagsearch(const char *pattern, const char *cwd, const char *root, const char *dbpath, int db)
 {
-	const char *p;
-	int length;
+	int count, total = 0;
+	char libdbpath[MAXPATHLEN+1];
 
-	if (!(p = locatestring(line, "./", MATCH_FIRST)))
-		die("invalid tag format (path not found).");
-	length = strlen(path);
-	if (strncmp(p, path, length))
-		return 0;
-	p += length;
-	if (*p == ' ' || *p == '\t')
-		return 1;
-	return 0;
+	/*
+	 * search in current source tree.
+	 */
+	count = search(pattern, root, cwd, dbpath, db);
+	total += count;
+	/*
+	 * search in library path.
+	 */
+	if (getenv("GTAGSLIBPATH") && (count == 0 || Tflag) && !lflag && !rflag) {
+		STRBUF *sb = strbuf_open(0);
+		char *libdir, *nextp = NULL;
+
+		strbuf_puts(sb, getenv("GTAGSLIBPATH"));
+		/*
+		 * search for each tree in the library path.
+		 */
+		for (libdir = strbuf_value(sb); libdir; libdir = nextp) {
+			if ((nextp = locatestring(libdir, PATHSEP, MATCH_FIRST)) != NULL)
+				*nextp++ = 0;
+			if (!gtagsexist(libdir, libdbpath, sizeof(libdbpath), 0))
+				continue;
+			if (!strcmp(dbpath, libdbpath))
+				continue;
+			if (!test("f", makepath(libdbpath, dbname(db), NULL)))
+				continue;
+
+			/*
+			 * reconstruct path filter
+			 */
+			setup_pathfilter(format, aflag ? PATH_ABSOLUTE : PATH_RELATIVE, libdir, cwd, libdbpath);
+			/*
+			 * search again
+			 */
+			count = search(pattern, libdir, cwd, libdbpath, db);
+			total += count;
+			if (count > 0 && !Tflag) {
+				/* for verbose message */
+				dbpath = libdbpath;
+				break;
+			}
+		}
+		strbuf_close(sb);
+	}
+	if (vflag) {
+		switch (total) {
+		case 0:
+			fprintf(stderr, "'%s' not found", pattern);
+			break;
+		case 1:
+			fprintf(stderr, "%d object located", total);
+			break;
+		default:
+			fprintf(stderr, "%d objects located", total);
+			break;
+		}
+		if (!Tflag)
+			fprintf(stderr, " (using '%s')", makepath(dbpath, dbname(db), NULL));
+		fputs(".\n", stderr);
+	}
 }
 /*
- * ffformat: string copy with converting blank chars into %ff format.
+ * encode: string copy with converting blank chars into %ff format.
  *
  *	o)	to	result
  *	i)	size	size of 'to' buffer
  *	i)	from	string
  */
 void
-ffformat(to, size, from)
-	char *to;
-	int size;
-	const char *from;
+encode(char *to, int size, const char *from)
 {
 	const char *p;
 	char *e = to;
