@@ -43,6 +43,7 @@
 #include "getopt.h"
 #include "regex.h"
 #include "global.h"
+#include "anchor.h"
 #include "cache.h"
 #include "common.h"
 #include "htags.h"
@@ -77,10 +78,9 @@ char distpath[MAXPATHLEN];
 char gtagsconf[MAXPATHLEN];
 char datadir[MAXPATHLEN];
 
-char sort_path[MAXFILLEN];
 char gtags_path[MAXFILLEN];
 char global_path[MAXFILLEN];
-char findcom[MAXFILLEN];
+int gtags_exist[GTAGLIM];
 const char *null_device = NULL_DEVICE;
 const char *tmpdir = "/tmp";
 
@@ -103,7 +103,7 @@ int show_help;				/* --help command		*/
 int show_version;			/* --version command		*/
 int caution;				/* --caution option		*/
 int dynamic;				/* --dynamic(-D) option		*/
-int symbol;				/* --symbol(-s) option		*/
+int symbol;				/* --symbol(-s) option          */
 int statistics;				/* --statistics option		*/
 
 int copy_files;				/* 1: copy tag files		*/
@@ -118,7 +118,6 @@ const char *action_value;
 const char *id_value;
 const char *cgidir;
 const char *main_func = "main";
-const char *style_sheet;
 const char *cvsweb_url;
 const char *cvsweb_cvsroot;
 const char *gtagslabel;
@@ -186,7 +185,6 @@ const char *file_icon = "text";
  */
 int ncol = 4;				/* columns of line number	*/
 int tabs = 8;				/* tab skip			*/
-char stabs[8];				/* tab skip (string)		*/
 int full_path = 0;			/* file index format		*/
 int map_file = 1;			/* 1: create MAP file		*/
 const char *icon_list = NULL;		/* use icon list		*/
@@ -242,7 +240,6 @@ static struct option const long_options[] = {
         {"insert-header", required_argument, NULL, 1},
         {"insert-footer", required_argument, NULL, 1},
         {"statistics", no_argument, &statistics, 1},
-        {"style-sheet", required_argument, NULL, 1},
         {"version", no_argument, &show_version, 1},
         {"help", no_argument, &show_help, 1},
         { 0 }
@@ -277,8 +274,7 @@ clean(void)
  * This handler is set up in signal_setup().
  */
 static void
-suddenly(signo)
-        int signo;
+suddenly(int signo)
 {
         signo = 0;      /* to satisfy compiler */
 
@@ -309,9 +305,7 @@ signal_setup(void)
  *	i)	file	file name
  */
 static void
-generate_file(dist, file)
-	const char *dist;
-	const char *file;
+generate_file(const char *dist, const char *file)
 {
 	regex_t preg;
 	regmatch_t pmatch[2];
@@ -326,7 +320,6 @@ generate_file(dist, file)
 		/* dynamic initialization */
                 {"@page_begin@", NULL},
                 {"@page_end@", NULL},
-                {"@lineno_anchor@", NULL},
 
 		/* static initialization */
                 {"@body_begin@", body_begin},
@@ -339,10 +332,7 @@ generate_file(dist, file)
                 {"@message_end@", message_end},
                 {"@verbatim_begin@", verbatim_begin},
                 {"@verbatim_end@", verbatim_end},
-                {"@global_path@", global_path},
-                {"@gtags_path@", gtags_path},
                 {"@normal_suffix@", normal_suffix},
-                {"@tabs@", stabs},
                 {"@hr@", hr},
                 {"@br@", br},
                 {"@HTML@", HTML},
@@ -355,7 +345,6 @@ generate_file(dist, file)
 
 	tab[0].value = gen_page_begin("Result", SUBDIR);
 	tab[1].value = gen_page_end();
-	tab[2].value = gen_name_string("L$.");
 	/*
 	 * construct regular expression.
 	 */
@@ -427,9 +416,7 @@ generate_file(dist, file)
  * makeprogram: make CGI program
  */
 static void
-makeprogram(cgidir, file)
-	const char *cgidir;
-	const char *file;
+makeprogram(const char *cgidir, const char *file)
 {
 	generate_file(cgidir, file);
 }
@@ -437,8 +424,7 @@ makeprogram(cgidir, file)
  * makebless: make bless.sh file.
  */
 static void
-makebless(file)
-	const char *file;
+makebless(const char *file)
 {
 	const char *save = action;
 	action = saction;
@@ -452,9 +438,7 @@ makebless(file)
  *	i)	file	file name
  */
 static void
-makeghtml(cgidir, file)
-	const char *cgidir;
-	const char *file;
+makeghtml(const char *cgidir, const char *file)
 {
 	FILE *op;
 
@@ -471,8 +455,7 @@ makeghtml(cgidir, file)
  * makerebuild: make rebuild script
  */
 static void
-makerebuild(file)
-	const char *file;
+makerebuild(const char *file)
 {
 	FILE *op;
 
@@ -493,8 +476,7 @@ makerebuild(file)
  * makehelp: make help file
  */
 static void
-makehelp(file)
-	const char *file;
+makehelp(const char *file)
 {
 	const char **label = icon_list ? anchor_comment : anchor_label;
 	const char **icons = anchor_icons;
@@ -510,7 +492,8 @@ makehelp(file)
 	fputs(header_begin, op);
 	fputs("Usage of Links", op);
 	fputs_nl(header_end, op);
-	fputs(verbatim_begin, op);
+	if (!icon_list)
+		fputs(verbatim_begin, op);
 	fputs("/* ", op);
 	for (n = 0; n <= last; n++) {
 		if (icon_list) {
@@ -524,7 +507,10 @@ makehelp(file)
 	if (show_position)
 		fprintf(op, "[+line file]");
 	fputs(" */", op);
-	fputs_nl(verbatim_end, op);
+	if (!icon_list)
+		fputs_nl(verbatim_end, op);
+	else
+		fputc('\n', op);
 	fputs_nl(define_list_begin, op);
 	for (n = 0; n <= last; n++) {
 		fputs(define_term_begin, op);
@@ -561,10 +547,7 @@ makehelp(file)
  *	r)		html
  */
 static char *
-makesearchpart(action, id, target)
-	const char *action;
-	const char *id;
-	const char *target;
+makesearchpart(const char *action, const char *id, const char *target)
 {
 	STATIC_STRBUF(sb);
 
@@ -624,10 +607,7 @@ makesearchpart(action, id, target)
  *	i)	index	common part
  */
 static void
-makeindex(file, title, index)
-	const char *file;
-	const char *title;
-	const char *index;
+makeindex(const char *file, const char *title, const char *index)
 {
 	FILE *op;
 
@@ -635,7 +615,7 @@ makeindex(file, title, index)
 	if (!op)
 		die("cannot make file '%s'.", file);
 	if (Fflag) {
-		fputs_nl(gen_frameset_page_begin(title), op);
+		fputs_nl(gen_page_frameset_begin(title), op);
 		fputs_nl(gen_frameset_begin("cols='200,*'"), op);
 		if (fflag) {
 			fputs_nl(gen_frameset_begin("rows='33%,33%,*'"), op);
@@ -679,9 +659,7 @@ makeindex(file, title, index)
  *	i)	index	common part
  */
 static void
-makemainindex(file, index)
-	const char *file;
-	const char *index;
+makemainindex(const char *file, const char *index)
 {
 	FILE *op;
 
@@ -706,8 +684,7 @@ makemainindex(file, index)
  *	i)	file	file name
  */
 static void
-makesearchindex(file)
-	const char *file;
+makesearchindex(const char *file)
 {
 	FILE *op;
 
@@ -726,8 +703,7 @@ makesearchindex(file)
  * makehtaccess: make .htaccess skeleton file.
  */
 static void
-makehtaccess(file)
-	const char *file;
+makehtaccess(const char *file)
 {
 	FILE *op;
 
@@ -755,56 +731,57 @@ makehtaccess(file)
  *	i)	total	number of files.
  */
 static void
-makehtml(total)
-	int total;
+makehtml(int total)
 {
-	FILE *ip;
-	const char *_;
+	GFIND *gp;
+	FILE *anchor_stream;
+	const char *path;
 	int count = 0;
-	char command[MAXFILLEN], path[MAXPATHLEN];
-	STRBUF *sb = strbuf_open(0);
 
-	if (other_files && !dynamic)
-		snprintf(command, sizeof(command), "%s --other | gnusort -t / -k 2", findcom);
-	else
-		strlimcpy(command, findcom, sizeof(command));
-	ip = popen(command, "r");
-	if (!ip)
-		die("cannot execute command '%s'.", command);
-	while ((_ = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
-		int notsource = 0;
-		const char *p;
-
-		if (*_ == ' ') {
-			if (!other_files)
-				continue;
-			_++;
-			if (test("b", _)) {
-				if (wflag)
-					warning("'%s' is binary file. (skipped)", _);
-				continue;
-			}
-			notsource = 1;
-		}
-		count++;
-		_ += 2;
-		p = _;
-		message(" [%d/%d] converting %s", count, total, p);
-		p = path2fid(p);
-		snprintf(path, sizeof(path), "%s/%s/%s.%s", distpath, SRCS, p, HTML);
-		src2html(_, path, notsource);
+	/*
+	 * Create anchor stream for anchor_load().
+	 */
+	anchor_stream = tmpfile();
+	gp = gfind_open(dbpath, NULL, other_files);
+	while ((path = gfind_read(gp)) != NULL) {
+		if (gp->type == GPATH_OTHER)
+			fputc(' ', anchor_stream);
+		fputs(path, anchor_stream);
+		fputc('\n', anchor_stream);
 	}
-	if (pclose(ip) != 0)
-		die("cannot traverse directory.(%s)", command);
-	strbuf_close(sb);
+	gfind_close(gp);
+	/*
+	 * Prepare anchor stream for anchor_load().
+	 */
+	anchor_prepare(anchor_stream);
+	/*
+	 * For each path in GPATH, convert the path into HTML file.
+	 */
+	gp = gfind_open(dbpath, NULL, other_files);
+	while ((path = gfind_read(gp)) != NULL) {
+		char html[MAXPATHLEN];
+
+		if (gp->type == GPATH_OTHER && !other_files)
+			continue;
+		/*
+		 * load tags belonging to the path.
+		 * The path must be start "./".
+		 */
+		anchor_load(path);
+
+		count++;
+		path += 2;		/* remove './' at the head */
+		message(" [%d/%d] converting %s", count, total, path);
+		snprintf(html, sizeof(html), "%s/%s/%s.%s", distpath, SRCS, path2fid(path), HTML);
+		src2html(path, html, gp->type == GPATH_OTHER);
+	}
+	gfind_close(gp);
 }
 /*
  * copy file.
  */
 static void
-copyfile(from, to)
-	const char *from;
-	const char *to;
+copyfile(const char *from, const char *to)
 {
 	int ip, op, size;
 	char buf[8192];
@@ -824,28 +801,6 @@ copyfile(from, to)
 	close(op);
 	close(ip);
 }
-/*
- * duplicate file.
- * By default, htags uses link system call without making a copy.
- */
-static void
-duplicatefile(file, from, to)
-	const char *file;
-	const char *from;
-	const char *to;
-{
-	char from_path[MAXPATHLEN];
-	char to_path[MAXPATHLEN];
-
-	snprintf(from_path, sizeof(from_path), "%s/%s", from, file);
-	snprintf(to_path, sizeof(to_path), "%s/%s", to, file);
-        if (copy_files) {
-                copyfile(from_path, to_path);
-	} else {
-                if (link(from_path, to_path) < 0)
-                        copyfile(from_path, to_path);
-	}
-}
 /*
  * makecommonpart: make a common part for mains.html and index.html
  *
@@ -855,10 +810,7 @@ duplicatefile(file, from, to)
  *	r)	index	common part
  */
 static char *
-makecommonpart(title, defines, files)
-	const char *title;
-	const char *defines;
-	const char *files;
+makecommonpart(const char *title, const char *defines, const char *files)
 {
 	FILE *ip;
 	STRBUF *sb = strbuf_open(0);
@@ -898,7 +850,7 @@ makecommonpart(title, defines, files)
 	}
 	strbuf_sprintf(sb, "%sMAINS%s\n", header_begin, header_end);
 
-	snprintf(command, sizeof(command), "%s -nx %s | gnusort -k 1,1 -k 3,3 -k 2,2n", global_path, main_func);
+	snprintf(command, sizeof(command), "%s -x --nofilter=path %s", global_path, main_func);
         ip = popen(command, "r");
         if (!ip)
                 die("cannot execute command '%s'.", command);
@@ -959,18 +911,12 @@ basic_check(void)
 	/*
 	 * COMMAND EXISTENCE CHECK
 	 */
-	if (!(p = usable("gnusort")))
-		die("gnusort command required but not found.");
-	strlimcpy(sort_path, p, sizeof(sort_path));
 	if (!(p = usable("gtags")))
 		die("gtags command required but not found.");
 	strlimcpy(gtags_path, p, sizeof(gtags_path));
 	if (!(p = usable("global")))
 		die("global command required but not found.");
 	strlimcpy(global_path, p, sizeof(global_path));
-
-	snprintf(findcom, sizeof(findcom), "%s --find", gtags_path);
-
 	/*
 	 * Temporary directory.
 	 */
@@ -983,9 +929,7 @@ basic_check(void)
  * load configuration variables.
  */
 static void
-configuration(argc, argv)
-	int argc;
-	char *argv[];
+configuration(int argc, char **argv)
 {
 	STRBUF *sb = strbuf_open(0);
 	int i, n;
@@ -999,28 +943,24 @@ configuration(argc, argv)
 		char *confpath = NULL;
 		char *label = NULL;
 		char *opt_gtagsconf = "--gtagsconf";
-		int leng_gtagsconf = strlen(opt_gtagsconf);
 		char *opt_gtagslabel = "--gtagslabel";
-		int leng_gtagslabel = strlen(opt_gtagslabel);
 
 		for (i = 1; i < argc; i++) {
-			if (!strncmp(argv[i], opt_gtagsconf, leng_gtagsconf)) {
-				if (!strcmp(argv[i], opt_gtagsconf)) {
+			if ((p = locatestring(argv[i], opt_gtagsconf, MATCH_AT_FIRST))) {
+				if (*p == '\0') {
 					if (++i >= argc)
 						die("%s needs an argument.", opt_gtagsconf);
 					confpath = argv[i];
 				} else {
-					p = argv[i] + leng_gtagsconf;
 					if (*p++ == '=' && *p)
 						confpath = p;
 				}
-			} else if (!strncmp(argv[i], opt_gtagslabel, leng_gtagslabel)) {
-				if (!strcmp(argv[i], opt_gtagslabel)) {
+			} else if ((p = locatestring(argv[i], opt_gtagslabel, MATCH_AT_FIRST))) {
+				if (*p == '\0') {
 					if (++i >= argc)
 						die("%s needs an argument.", opt_gtagslabel);
 					label = argv[i];
 				} else {
-					p = argv[i] + leng_gtagslabel;
 					if (*p++ == '=' && *p)
 						label = p;
 				}
@@ -1057,7 +997,6 @@ configuration(argc, argv)
 		else
 			tabs = n;
 	}
-	snprintf(stabs, sizeof(stabs), "%d", tabs);
 	strbuf_reset(sb);
 	if (getconfs("gzipped_suffix", sb)) {
 		p = strdup(strbuf_value(sb));
@@ -1150,10 +1089,6 @@ configuration(argc, argv)
 			*q = '\0';
 		script_alias = p;
 	}
-	if (getconfb("symbols"))	/* for backward compatibility */
-		symbol = 1;
-	if (getconfb("symbol"))
-		symbol = 1;
 	if (getconfb("dynamic"))
 		dynamic = 1;
 	strbuf_reset(sb);
@@ -1335,9 +1270,7 @@ configuration(argc, argv)
  * save_environment: save configuration data and arguments.
  */
 static void
-save_environment(argc, argv)
-	int argc;
-	char *argv[];
+save_environment(int argc, char **argv)
 {
 	char command[MAXFILLEN];
 	STRBUF *sb = strbuf_open(0);
@@ -1377,7 +1310,6 @@ save_environment(argc, argv)
 	 */
 	{
 		char *opt_gtagsconf = "--gtagsconf";
-		int leng_gtagsconf = strlen(opt_gtagsconf);
 
 		for (i = 1; i < argc; i++) {
 			char *blank;
@@ -1386,8 +1318,8 @@ save_environment(argc, argv)
 			 * skip --gtagsconf because it is already read
 			 * as config value.
 			 */
-			if (!strncmp(argv[i], opt_gtagsconf, leng_gtagsconf)) {
-				if (!strcmp(argv[i], opt_gtagsconf))
+			if ((p = locatestring(argv[i], opt_gtagsconf, MATCH_AT_FIRST))) {
+				if (*p == '\0')
 					i++;
 				continue;
 			}
@@ -1406,9 +1338,7 @@ save_environment(argc, argv)
 }
 
 char **
-append_options(argc, argv)
-	int *argc;
-	char *argv[];
+append_options(int *argc, char **argv)
 {
 
 	STRBUF *sb = strbuf_open(0);
@@ -1467,9 +1397,7 @@ append_options(argc, argv)
 	return argv;
 }
 int
-main(argc, argv)
-        int argc;
-        char *argv[];
+main(int argc, char **argv)
 {
 	const char *path, *av = NULL;
 	int func_total, file_total;
@@ -1511,8 +1439,6 @@ main(argc, argv)
 				;	/*  --gtagsconf is estimated only once. */
 			else if (!strcmp("gtagslabel", long_options[option_index].name))
 				;	/* --gtagslabel is estimated only once. */
-			else if (!strcmp("style-sheet", long_options[option_index].name))
-                                style_sheet = optarg;
 			else if (!strcmp("insert-header", long_options[option_index].name))
 				insert_header = optarg;
 			else if (!strcmp("insert-footer", long_options[option_index].name))
@@ -1650,22 +1576,6 @@ main(argc, argv)
 	}
 	if (dynamic && Sflag)
 		die("Current implementation doesn't allow both -D(--dynamic) and the -S(--secure-cgi).");
-	if (style_sheet) {
-		char buf[MAXBUFLEN];
-		STRBUF *sb = strbuf_open(0);
-		FILE *ip = fopen(style_sheet, "r");
-
-		if (ip) {
-			while (fgets(buf, sizeof(buf), ip)) {
-				if (*buf != '\t' && *buf != '<')
-					strbuf_putc(sb, '\t');
-				strbuf_puts(sb, buf);
-			}
-			fclose(ip);
-		};
-		style_sheet = strbuf_value(sb);
-		/* Doesn't close string buffer. */
-	}
 	if (icon_list && !test("f", icon_list))
 		die("icon_list '%s' not found.", icon_list);
 	/*
@@ -1702,10 +1612,31 @@ main(argc, argv)
 	if (id_value) {
 		id = id_value;
 	}
-	if (!test("r", makepath(dbpath, dbname(GTAGS), NULL)) || !test("r", makepath(dbpath, dbname(GRTAGS), NULL)))
-		die("GTAGS and/or GRTAGS not found. Htags needs both of them.");
-	if (symbol && !test("r", makepath(dbpath, dbname(GSYMS), NULL)))
-		die("-s(--symbol) option needs GSYMS tag file.");
+	/*
+	 * Existence check of tag files.
+	 */
+	{
+		int i;
+		char *path;
+		GTOP *gtop;
+
+		for (i = GPATH; i < GTAGLIM; i++) {
+			path = makepath(dbpath, dbname(i), NULL);
+			gtags_exist[i] = test("fr", path);
+		}
+		if (!gtags_exist[GPATH] || !gtags_exist[GTAGS])
+			die("GPATH and/or GTAGS not found. Please reexecute htags with the -g option.");
+		if (!symbol)
+			gtags_exist[GSYMS] = 0;
+		else if (!gtags_exist[GSYMS])
+			die("the -s(--symbol) option needs GSYMS. Please reexecute htags with the -g option.");
+		/*
+		 * version check.
+		 * Do nothing, but the version of tag file will be checked.
+		 */
+		gtop = gtags_open(dbpath, cwdpath, GTAGS, GTAGS_READ);
+		gtags_close(gtop);
+	}
 	/*
 	 * make dbpath absolute.
 	 */
@@ -1715,6 +1646,16 @@ main(argc, argv)
 			die("cannot get realpath of dbpath.");
 		strlimcpy(dbpath, buf, sizeof(dbpath));
 	}
+	/*
+	 * The older version (4.8.7 or former) of GPATH doesn't have files
+         * other than source file. The oflag requires new version of GPATH.
+	 */
+	if (other_files) {
+		GFIND *gp = gfind_open(dbpath, NULL, 0);
+		if (gp->version < 2)
+			die("GPATH is old format. Please remake it by invoking gtags(1).");
+		gfind_close(gp);
+	}
 	/*
 	 * for global(1) and gtags(1).
 	 */
@@ -1794,9 +1735,8 @@ main(argc, argv)
 			if (dynamic)
 				if (!strcmp(p, DEFS) || !strcmp(p, REFS) || !strcmp(p, SYMS))
 					continue;
-			if (!symbol)
-				if (!strcmp(p, SYMS))
-					continue;
+			if (!symbol && !strcmp(p, SYMS))
+				continue;
 			path = makepath(distpath, p, NULL);
 			if (!test("d", path))
 				if (mkdir(path, mode))
@@ -1813,8 +1753,6 @@ main(argc, argv)
 	 * (1) make CGI program
 	 */
 	if (cgi && (fflag || dynamic)) {
-		int i;
-
 		if (cgidir) {
 			message("[%s] (1) making CGI program ...", now());
 			makeprogram(cgidir, "global.cgi");
@@ -1828,21 +1766,6 @@ main(argc, argv)
 		makebless("bless.sh");
 		if (chmod(makepath(distpath, "bless.sh", NULL), 0640) < 0)
 			die("cannot chmod bless script.");
-		/*
-		 * replace G*** tag files.
-		 */
-		for (i = GPATH; i < GTAGLIM; i++) {
-			const char *name = dbname(i);
-
-			if (test("f", makepath(dbpath, name, NULL))) {
-				char path[MAXPATHLEN];
-
-				snprintf(path, sizeof(path), "%s/cgi-bin/%s", distpath, name);
-				unlink(path);
-				snprintf(path, sizeof(path), "%s/cgi-bin", distpath);
-				duplicatefile(name, dbpath, path);
-			}
-		}
 	} else {
 		message("[%s] (1) making CGI program ...(skipped)", now());
 	}
@@ -1967,7 +1890,7 @@ main(argc, argv)
 	end_all_time = time(NULL);
 	message("[%s] Done.", now());
 	T_all = end_all_time - start_all_time;
-	if (vflag && cgi && (cflag || fflag)) {
+	if (vflag && cgi && (cflag || fflag || dynamic)) {
 		message("\n[Information]\n");
 		if (cflag) {
 			message(" Your system may need to be setup to decompress *.%s files.", gzipped_suffix);
@@ -1976,10 +1899,14 @@ main(argc, argv)
 			message(" gzipped files. (Please see 'HTML/.htaccess')\n");
 		}
 		if (fflag || dynamic) {
-			const char *path = (*action == '/') ? makepath("DOCUMENT_ROOT", action, NULL) : makepath("HTML", action, NULL);
+			const char *format = " You need to setup http server so that %s is executed as a CGI script.";
 
-			message(" You need to setup http server so that %s", path);
-			message(" is executed as a CGI script. (DOCUMENT_ROOT means WWW server's data root.)\n");
+			if (*action == '/') {
+				message(format, makepath("DOCUMENT_ROOT", action, NULL));
+				message(" (DOCUMENT_ROOT means WWW server's data root.)");
+			} else {
+				message(format, makepath("HTML", action, NULL));
+			}
 		}
 		message(" Good luck!\n");
 	}
