@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+ *		2006
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -91,8 +92,7 @@ struct lang_entry lang_switch[] = {
  *	r)		language entry
  */
 static struct lang_entry *
-get_lang_entry(lang)
-	const char *lang;
+get_lang_entry(const char *lang)
 {
 	int i, size = sizeof(lang_switch) / sizeof(struct lang_entry);
 
@@ -130,16 +130,13 @@ static int last_lineno;
  *	r)		file pointer
  */
 static FILE *
-open_input_file(file)
-	const char *file;
+open_input_file(const char *file)
 {
-	char command[MAXFILLEN];
 	FILE *ip;
 
-	snprintf(command, sizeof(command), "gtags --expand -%d < %s", tabs, file);
-	ip = popen(command, "r");
+	ip = fopen(file, "r");
 	if (!ip)
-		die("cannot execute '%s'.", command);
+		die("cannot open file '%s'.", file);
 	curpfile = file;
 	warned = 0;
 	return ip;
@@ -148,11 +145,9 @@ open_input_file(file)
  * Close source file.
  */
 static void
-close_input_file(ip)
-	FILE *ip;
+close_input_file(FILE *ip)
 {
-	if (pclose(ip) != 0)
-		die("command 'gtags --expand -%d' failed.", tabs);
+	fclose(ip);
 }
 /*
  * Open HTML file.
@@ -161,8 +156,7 @@ close_input_file(ip)
  *	r)		file pointer
  */
 static FILE *
-open_output_file(file)
-	const char *file;
+open_output_file(const char *file)
 {
 	char command[MAXFILLEN];
 	FILE *op;
@@ -184,8 +178,7 @@ open_output_file(file)
  * Close HTML file.
  */
 static void
-close_output_file(op)
-	FILE *op;
+close_output_file(FILE *op)
 {
 	if (cflag) {
 		if (pclose(op) != 0)
@@ -199,8 +192,7 @@ close_output_file(op)
  * You should use this function to put a control character.
  */
 void
-echoc(c)
-	int c;
+echoc(int c)
 {
         strbuf_putc(outbuf, c);
 }
@@ -210,8 +202,7 @@ echoc(c)
  * You should use this function to put a control sequence.
  */
 void
-echos(s)
-	const char *s;
+echos(const char *s)
 {
         strbuf_puts(outbuf, s);
 }
@@ -227,9 +218,7 @@ echos(s)
  *       r)              hypertext file name string
  */
 const char *
-fill_anchor(root, path)
-	const char *root;
-	const char *path;
+fill_anchor(const char *root, const char *path)
 {
 	STATIC_STRBUF(sb);
 	char buf[MAXBUFLEN], *limit, *p;
@@ -273,8 +262,7 @@ fill_anchor(root, path)
  *	r)	HTML
  */
 const char *
-link_format(ref)
-        int ref[A_SIZE];
+link_format(int ref[A_SIZE])
 {
 	STATIC_STRBUF(sb);
 	const char **label = icon_list ? anchor_comment : anchor_label;
@@ -317,8 +305,7 @@ link_format(ref)
  *	r)		guide string
  */
 const char *
-generate_guide(lineno)
-	int lineno;
+generate_guide(int lineno)
 {
 	STATIC_STRBUF(sb);
 	int i = 0;
@@ -349,10 +336,7 @@ generate_guide(lineno)
  *	r)		tooltip string
  */
 const char *
-tooltip(type, lno, opt)
-	int type;
-	int lno;
-	const char *opt;
+tooltip(int type, int lno, const char *opt)
 {
 	STATIC_STRBUF(sb);
 
@@ -398,10 +382,7 @@ tooltip(type, lno, opt)
  *	i)	lineno	current line no
  */
 void
-put_anchor(name, type, lineno)
-	char *name;
-	int type;
-	int lineno;
+put_anchor(char *name, int type, int lineno)
 {
 	const char *line;
 	int db;
@@ -477,9 +458,7 @@ put_anchor(name, type, lineno)
  *	i)	path	path name for display
  */
 void
-put_include_anchor(inc, path)
-	struct data *inc;
-	const char *path;
+put_include_anchor(struct data *inc, const char *path)
 {
 	if (inc->count == 1)
 		strbuf_puts(outbuf, gen_href_begin(NULL, path2fid(strbuf_value(inc->contents)), HTML, NULL));
@@ -495,8 +474,7 @@ put_include_anchor(inc, path)
  * Put a reserved word. (if, while, ...)
  */
 void
-put_reserved_word(word)
-	const char *word;
+put_reserved_word(const char *word)
 {
 	strbuf_puts(outbuf, reserved_begin);
 	strbuf_puts(outbuf, word);
@@ -506,8 +484,7 @@ put_reserved_word(word)
  * Put a macro (#define,#undef,...) 
  */
 void
-put_macro(word)
-	const char *word;
+put_macro(const char *word)
 {
 	strbuf_puts(outbuf, sharp_begin);
 	strbuf_puts(outbuf, word);
@@ -517,9 +494,7 @@ put_macro(word)
  * Print warning message when unkown preprocessing directive is found.
  */
 void
-unknown_preprocessing_directive(word, lineno)
-	const char *word;
-	int lineno;
+unknown_preprocessing_directive(const char *word, int lineno)
 {
 	word = strtrim(word, TRIM_ALL, NULL);
 	warning("unknown preprocessing directive '%s'. [+%d %s]", word, lineno, curpfile);
@@ -530,8 +505,7 @@ unknown_preprocessing_directive(word, lineno)
  * Print warning message when unexpected eof.
  */
 void
-unexpected_eof(lineno)
-	int lineno;
+unexpected_eof(int lineno)
 {
 	warning("unexpected eof. [+%d %s]", lineno, curpfile);
 	if (colorize_warned_line)
@@ -541,9 +515,7 @@ unexpected_eof(lineno)
  * Print warning message when unknown yacc directive is found.
  */
 void
-unknown_yacc_directive(word, lineno)
-	const char *word;
-	int lineno;
+unknown_yacc_directive(const char *word, int lineno)
 {
 	warning("unknown yacc directive '%s'. [+%d %s]", word, lineno, curpfile);
 	if (colorize_warned_line)
@@ -553,9 +525,7 @@ unknown_yacc_directive(word, lineno)
  * Print warning message when unmatched brace is found.
  */
 void
-missing_left(word, lineno)
-	const char *word;
-	int lineno;
+missing_left(const char *word, int lineno)
 {
 	warning("missing left '%s'. [+%d %s]", word, lineno, curpfile);
 	if (colorize_warned_line)
@@ -567,8 +537,7 @@ missing_left(word, lineno)
  * If you want to put '<', '>' and '&', you should echoc() instead.
  */
 void
-put_char(c)
-        int c;
+put_char(int c)
 {
         if (c == '<')
 		strbuf_puts(outbuf, quote_little);
@@ -585,8 +554,7 @@ put_char(c)
  * If you want to put HTML tag itself, you should echoc() instead.
  */
 void
-put_string(s)
-        const char *s;
+put_string(const char *s)
 {
 	for (; *s; s++)
 		put_char(*s);
@@ -595,8 +563,7 @@ put_string(s)
  * Put brace ('{', '}')
  */
 void
-put_brace(text)
-        const char *text;
+put_brace(const char *text)
 {
 	strbuf_puts(outbuf, brace_begin);
 	strbuf_puts(outbuf, text);
@@ -613,8 +580,7 @@ static const char *guide = NULL;
  * Begin of line processing.
  */
 void
-put_begin_of_line(lineno)
-        int lineno;
+put_begin_of_line(int lineno)
 {
         if (definition_header != NO_HEADER) {
                 if (define_line(lineno))
@@ -637,8 +603,7 @@ put_begin_of_line(lineno)
  * This function flush and clear it.
  */
 void
-put_end_of_line(lineno)
-	int lineno;
+put_end_of_line(int lineno)
 {
 	fputs(gen_name_number(lineno), out);
         if (nflag)
@@ -677,10 +642,7 @@ put_end_of_line(lineno)
  *       i)      notsource 1: isn't source, 0: source.
  */
 void
-src2html(src, html, notsource)
-	const char *src;
-	const char *html;
-	int notsource;
+src2html(const char *src, const char *html, int notsource)
 {
 	char indexlink[128];
 
@@ -696,12 +658,6 @@ src2html(src, html, notsource)
 		snprintf(indexlink, sizeof(indexlink), "../files.%s", normal_suffix);
 	else
 		snprintf(indexlink, sizeof(indexlink), "../mains.%s", normal_suffix);
-	/*
-	 * load tags belonging to this file.
-	 */
-	if (!notsource) {
-		anchor_load(src);
-	}
 	fputs_nl(gen_page_begin(src, SUBDIR), out);
 	fputs_nl(body_begin, out);
 	/*
@@ -756,18 +712,27 @@ src2html(src, html, notsource)
 		fputs_nl(verbatim_begin, out);
 		last_lineno = 0;
 		while ((_ = strbuf_fgets(sb, in, STRBUF_NOCRLF)) != NULL) {
+			int dst = 0;
+
 			fputs(gen_name_number(++last_lineno), out);
 			for (; *_; _++) {
 				int c = *_;
 
-				if (c == '&')
-					fputs(quote_amp, out);
-				else if (c == '<')
-					fputs(quote_little, out);
-				else if (c == '>')
-					fputs(quote_great, out);
-				else
-					fputc(c, out);
+				if (c == '\t') {
+					do {
+						putc(' ', out);
+					} while (++dst % tabs);
+				} else {
+					if (c == '&')
+						fputs(quote_amp, out);
+					else if (c == '<')
+						fputs(quote_little, out);
+					else if (c == '>')
+						fputs(quote_great, out);
+					else
+						fputc(c, out);
+					dst++;
+				}
 			}
 			fputc('\n', out);
 		}
@@ -797,10 +762,10 @@ src2html(src, html, notsource)
 			const char *dir, *file, *suffix, *key, *title;
 
 			fputs(header_begin, out);
-			if (incref->count > 1) {
+			if (incref->ref_count > 1) {
 				char s_count[32];
 
-				snprintf(s_count, sizeof(s_count), "%d", incref->count);
+				snprintf(s_count, sizeof(s_count), "%d", incref->ref_count);
 				snprintf(s_id, sizeof(s_id), "%d", incref->id);
 				dir = upperdir(INCREFS);
 				file = s_id;
@@ -808,7 +773,7 @@ src2html(src, html, notsource)
 				key = NULL;
 				title = tooltip('I', -1, s_count);
 			} else {
-				const char *p = strbuf_value(incref->contents);
+				const char *p = strbuf_value(incref->ref_contents);
 				const char *lno = strmake(p, " ");
 				const char *filename;
 
