@@ -32,25 +32,30 @@
 #include "die.h"
 #include "env.h"
 #include "find.h"
+#include "format.h"
 #include "getdbpath.h"
 #include "gtagsop.h"
+#include "idset.h"
 #include "is_unixy.h"
 #include "langmap.h"
 #include "linetable.h"
 #include "locatestring.h"
 #include "makepath.h"
 #include "path.h"
+#include "pathconvert.h"
 #include "gpathop.h"
 #include "split.h"
 #include "strbuf.h"
+#include "strhash.h"
 #include "strlimcpy.h"
 #include "strmake.h"
 #include "tab.h"
+#include "tagsort.h"
 #include "test.h"
 #include "token.h"
 #include "usable.h"
 #include "version.h"
 #include "varray.h"
-#include "vfind.h"
+#include "xargs.h"
 
 #endif /* ! _GLOBAL_H_ */
