@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2005
+ * Copyright (c) 1997, 1998, 1999, 2000, 2002, 2005, 2006
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -92,11 +92,11 @@ STRBUF	top;
  * strbuf_dump: dump string buffers
  */
 void
-strbuf_dump(msg)
-	const char *msg;
+strbuf_dump(const char *msg)
 {
 	STRBUF *sb;
 	int i = 0;
+	long total = 0, used = 0;
 
 	fprintf(stderr, "[%s/%s]\n", progname, msg);
 	for (sb = top.next; sb && sb != &top; sb = sb->next) {
@@ -104,15 +104,14 @@ strbuf_dump(msg)
 		char *end = p + strbuf_getlen(sb);
 
 		*sb->curp = 0;
-		fprintf(stderr, "%d\tsize=%d", strbuf_getlen(sb));
-		if (strbuf_getlen(sb) <= strlen(p))
-			fprintf(stderr, ", value=|%s|\n", p);
-		else {
-			fputc('\n', stderr);
-			for (; p < end; p += strlen(p) + 1)
-				fprintf(stderr, "\t|%s|", p);
-		}
+		fprintf(stderr, "(%d)\tsize=%d used=%d ", ++i, sb->sbufsize, strbuf_getlen(sb));
+		total += sb->sbufsize;
+		used += strbuf_getlen(sb);
+		for (; p < end; p += strlen(p) + 1)
+			fprintf(stderr, "[%s]", p);
+		fputc('\n', stderr);
 	}
+	fprintf(stderr, "Total %ld bytes, used %ld bytes\n", total, used);
 }
 #endif
 /*
@@ -122,9 +121,7 @@ strbuf_dump(msg)
  *	i)	length	required room
  */
 void
-__strbuf_expandbuf(sb, length)
-	STRBUF *sb;
-	int length;
+__strbuf_expandbuf(STRBUF *sb, int length)
 {
 	int count = sb->curp - sb->sbuf;
 	int newsize = sb->sbufsize + (length > EXPANDSIZE ? length : EXPANDSIZE);
@@ -152,8 +149,7 @@ __strbuf_expandbuf(sb, length)
  *	r)	sb	STRBUF structure
  */
 STRBUF *
-strbuf_open(init)
-	int init;
+strbuf_open(int init)
 {
 	STRBUF *sb = (STRBUF *)calloc(sizeof(STRBUF), 1);
 
@@ -183,6 +179,17 @@ strbuf_open(init)
 #endif
 	return sb;
 }
+/*
+ * strbuf_reset: reset string buffer.
+ *
+ *	i)	sb	string buffer
+ */
+void
+strbuf_reset(STRBUF *sb)
+{
+	sb->curp = sb->sbuf;
+	sb->alloc_failed = 0;
+}
 /*
  * strbuf_clear: clear static string buffer.
  *
@@ -192,8 +199,7 @@ strbuf_open(init)
  * For the detail, see 'STATIC_STRBUF(sb)' macro in strbuf.h.
  */
 void
-strbuf_clear(sb)
-	STRBUF *sb;
+strbuf_clear(STRBUF *sb)
 {
 	if (sb == NULL)
 		die("NULL string buffer. (strbuf_clear)");
@@ -217,10 +223,7 @@ strbuf_clear(sb)
  *	i)	len	length of string
  */
 void
-strbuf_nputs(sb, s, len)
-	STRBUF *sb;
-	const char *s;
-	int len;
+strbuf_nputs(STRBUF *sb, const char *s, int len)
 {
 	if (!sb->alloc_failed && len > 0) {
 		if (sb->curp + len > sb->endp)
@@ -229,6 +232,23 @@ strbuf_nputs(sb, s, len)
 			*sb->curp++ = *s++;
 	}
 }
+/*
+ * strbuf_nputc: Put characters with length
+ *
+ *	i)	sb	statically defined string buffer
+ *	i)	c	character
+ *	i)	len	length of string
+ */
+void
+strbuf_nputc(STRBUF *sb, int c, int len)
+{
+	if (!sb->alloc_failed && len > 0) {
+		if (sb->curp + len > sb->endp)
+			__strbuf_expandbuf(sb, len);
+		while (len-- > 0)
+			*sb->curp++ = c;
+	}
+}
 /*
  * strbuf_puts: Put string
  *
@@ -236,9 +256,7 @@ strbuf_nputs(sb, s, len)
  *	i)	s	string
  */
 void
-strbuf_puts(sb, s)
-	STRBUF *sb;
-	const char *s;
+strbuf_puts(STRBUF *sb, const char *s)
 {
 	if (!sb->alloc_failed) {
 		while (*s) {
@@ -255,9 +273,7 @@ strbuf_puts(sb, s)
  *	i)	s	string
  */
 void
-strbuf_puts_nl(sb, s)
-	STRBUF *sb;
-	const char *s;
+strbuf_puts_nl(STRBUF *sb, const char *s)
 {
 	if (!sb->alloc_failed) {
 		while (*s) {
@@ -277,9 +293,7 @@ strbuf_puts_nl(sb, s)
  *	i)	n	number
  */
 void
-strbuf_putn(sb, n)
-	STRBUF *sb;
-	int n;
+strbuf_putn(STRBUF *sb, int n)
 {
 	if (n == 0) {
 		strbuf_putc(sb, '0');
@@ -305,9 +319,7 @@ strbuf_putn(sb, n)
  *	r)		0: do nothing, 1: removed
  */
 int
-strbuf_unputc(sb, c)
-	STRBUF *sb;
-	int c;
+strbuf_unputc(STRBUF *sb, int c)
 {
 	if (sb->curp > sb->sbuf && *(sb->curp - 1) == c) {
 		sb->curp--;
@@ -322,8 +334,7 @@ strbuf_unputc(sb, c)
  *	r)		string
  */
 char *
-strbuf_value(sb)
-	STRBUF *sb;
+strbuf_value(STRBUF *sb)
 {
 	*sb->curp = 0;
 	return sb->sbuf;
@@ -334,8 +345,7 @@ strbuf_value(sb)
  *	i)	sb	STRBUR structure
  */
 void
-strbuf_trim(sb)
-	STRBUF *sb;
+strbuf_trim(STRBUF *sb)
 {
 	char *p = sb->curp;
 
@@ -358,10 +368,7 @@ strbuf_trim(sb)
  * include '\r' and '\n'.
  */
 char *
-strbuf_fgets(sb, ip, flags)
-	STRBUF *sb;
-	FILE *ip;
-	int flags;
+strbuf_fgets(STRBUF *sb, FILE *ip, int flags)
 {
 	if (!(flags & STRBUF_APPEND))
 		strbuf_reset(sb);
@@ -390,7 +397,7 @@ strbuf_fgets(sb, ip, flags)
 	if (flags & STRBUF_NOCRLF) {
 		if (*(sb->curp - 1) == '\n')
 			*(--sb->curp) = 0;
-		if (*(sb->curp - 1) == '\r')
+		if (sb->curp > sb->sbuf && *(sb->curp - 1) == '\r')
 			*(--sb->curp) = 0;
 	}
 	return sb->sbuf;
@@ -401,25 +408,14 @@ strbuf_fgets(sb, ip, flags)
  *	i)	sb	STRBUF structure
  *	i)	s	similar to sprintf()
  *			Currently the following format is supported.
- *			%s, %d, %<number>d, %<number>s
+ *			%s, %d, %<number>d, %<number>s, %-<number>d, %-<number>s
  */
 void
-#ifdef HAVE_STDARG_H
 strbuf_sprintf(STRBUF *sb, const char *s, ...)
-#else
-strbuf_sprintf(sb, s, va_alist)
-	STRBUF *sb;
-	const char *s;
-	va_dcl
-#endif
 {
 	va_list ap;
 
-#ifdef HAVE_STDARG_H
 	va_start(ap, s);
-#else
-	va_start(ap);
-#endif
 	if (sb->alloc_failed)
 		return;
 	for (; *s; s++) {
@@ -438,7 +434,7 @@ strbuf_sprintf(sb, s, va_alist)
 		if (*s == '\0')
 			break;
 		if (*s == '%') {
-			int c = *++s;
+			int c = (unsigned char)*++s;
 			/*
 			 * '%%' means '%'.
 			 */
@@ -450,13 +446,17 @@ strbuf_sprintf(sb, s, va_alist)
 			 * we forward the job to snprintf(3).
 			 * o %<number>d
 			 * o %<number>s
+			 * o %-<number>d
+			 * o %-<number>s
 			 */
-			else if (isdigit(c)) {
+			else if (isdigit(c) || (c == '-' && isdigit((unsigned char)*(s + 1)))) {
 				char format[32], buf[1024];
 				int i = 0;
 
 				format[i++] = '%';
-				while (isdigit(*s))
+				if (c == '-')
+					format[i++] = *s++;
+				while (isdigit((unsigned char)*s))
 					format[i++] = *s++;
 				format[i++] = c = *s;
 				format[i] = '\0';
@@ -484,8 +484,7 @@ strbuf_sprintf(sb, s, va_alist)
  *	i)	sb	STRBUF structure
  */
 void
-strbuf_close(sb)
-	STRBUF	*sb;
+strbuf_close(STRBUF *sb)
 {
 #ifdef STRBUF_LINK
 	sb->prev->next = sb->next;
@@ -521,8 +520,7 @@ strbuf_open_tempbuf(void)
 	return sb;
 }
 void
-strbuf_release_tempbuf(sb)
-	STRBUF *sb;
+strbuf_release_tempbuf(STRBUF *sb)
 {
 	used = 0;
 }
@@ -534,9 +532,7 @@ strbuf_release_tempbuf(sb)
  *	i)	name	name
  */
 void
-strbuf_setname(sb, name)
-	STRBUF  *sb;
-	const char *name;
+strbuf_setname(STRBUF  *sb, const char *name)
 {
 	char *p = strdup(name);
 	if (p == NULL) {
@@ -555,8 +551,7 @@ strbuf_setname(sb, name)
  *	r)		STRBUF structure
  */
 STRBUF *
-strbuf_getbuf(name)
-	const char *name;
+strbuf_getbuf(const char *name)
 {
 	STRBUF *sb;
 
