@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2004 Tama Communications Corporation
+ * Copyright (c) 1998, 1999, 2000, 2004
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -15,8 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifndef _STRMAKE_H_
@@ -38,7 +38,7 @@
 		p++;							\
 } while (0)
 
-char *strmake(const char *, const char *);
-char *strtrim(const char *, int, int *);
+const char *strmake(const char *, const char *);
+const char *strtrim(const char *, int, int *);
 
 #endif /* ! _STRMAKE_H_ */
